# -*- Mode: cperl -*-
#--------------------------------------------------------------------
# Copyright (C) 2000, 2001, 2002 by MandrakeSoft.
# Chmouel Boudjnah <chmouel@mandrakesoft.com>.
#
# Redistribution of this file is permitted under the terms of the GNU
# Public License (GPL)
#--------------------------------------------------------------------
# $Id: common.pm,v 1.42 2003/09/05 03:15:54 quintela Exp $
#--------------------------------------------------------------------

package common;
use strict;

my $debug = 0;

my $sysconfig = $debug ? "./installkernel.sysconfig" : "/etc/sysconfig/installkernel";
my $sysconfig_theme = "/etc/sysconfig/bootsplash";

my $mdksub = "smp|enterprise|secure|linus|mosix|BOOT|custom|i686-up-4GB|p3-smp-64GB";

sub check_default_entry {
    # dont ask me why i need to do that :p
    my $foo = $_[0];
    my %entry = %$foo;
    my $foo1 = $_[1];
    my %main = %$foo1;

    # check if the default entry if there is an initrd pointing to the
    # right kernel.
    while (my $act = each %entry) {

	# find the default ?
	if ($act =~ /^$main{default}$/) {
	    # there is an initrd ?
	    if ($entry{$act}{initrd}) {

		my $initrd_version_of_entry;
		my $kernel_version_of_entry;

		#too much code :p but i see problems coming and more
		#different code between the two case

		my $initrd_of_entry = $entry{$act}{initrd};
		$initrd_of_entry = readlink ($initrd_of_entry) if -l $initrd_of_entry;

		#A mdk kernel so make sure to get only the mdk one
		if ($initrd_of_entry =~ /mdk/ and $initrd_of_entry =~ m,(/boot/)?initrd-win4lin-(.*)mdk($mdksub|.*)?\.img?, ) {
		    my $s = $3;
		    $initrd_version_of_entry = $2 . "mdk" if $2;
		    #<chmou dark-satanique mode>
		    #	That tricky since if it is a mdk kernel it should
		    #	match $mdksub so since here if it's don't
		    #	match we don't add to the
		    #	$initrd_version_of_entry which mean wrong name
		    #	and fail.
		    #</chmou dark-satanique mode>
		    $initrd_version_of_entry .= $s if $s =~ /$mdksub/;
		}

		my $kernel_of_entry = $entry{$act}{kernel};
		$kernel_of_entry = readlink ($kernel_of_entry) if -l $kernel_of_entry;

		#A mdk kernel so make sure to get only the mdk one
		if ($kernel_of_entry =~ /mdk/ and $kernel_of_entry =~ m,(/boot/)?win4lin-(.*)mdk($mdksub|.*)?, ) {
		    my $s = $3;
		    $kernel_version_of_entry = $2 . "mdk" if $2;
		    if ($s and $s !~ /$mdksub/) {
			print "Looks like you have an unknown mdk kernel, please contact Stew <sbenedict\@mandrakesoft.com>\n";
			# We still continue and don't make it fail
			# fail, since we want to add a new kernel
			# anyway
			return;
		    }
		    $kernel_version_of_entry .= $s if $s;
		}

		print "kernel=$kernel_version_of_entry, initrd=$initrd_version_of_entry\n" if $debug;
		if ($kernel_version_of_entry !~ /^$initrd_version_of_entry$/) {
		    print STDERR ("look like there was a problem, the default win4lin version is not the same \nof the initrd\n");
		    print STDERR ("which mean you have a mdk kernel and not a mdk initrd you may go in trouble\n");
		    return(1);
		}
	    } else { #check without initrd
		my $kernel_of_entry = $entry{$act}{kernel};
		$kernel_of_entry = readlink ($kernel_of_entry) if -l $kernel_of_entry;

		#A mdk kernel so make sure to get only the mdk one
		if ($kernel_of_entry =~ /mdk/ and $kernel_of_entry =~ m,(/boot/)?win4lin-(.*)mdk($mdksub|.*)?, ) {
		    my $s = $3;
		    if ($s and $s !~ /$mdksub/) {
			print "Looks like you have an unknown mdk kernel => $kernel_of_entry\n";
			print "please send a mail on Mandrake Cooker <cooker\@linux-mandrake.com>\n";
			# We still continue and don't make it fail
			# fail, since we want to add a new kernel
			# anyway
			return;
		    }
		}
	    }
	}
     }
}

sub get_first_boot {
    my ($type, $resolved_link) = @_;
    my $boot = '/boot/';
    my @list;
    opendir D, $boot;
    for (readdir(D)) {
	if (/win4lin-.*$/i and not /$resolved_link/ and not /win4lin-($mdksub)/) {
	    push @list, $_
	}
    }
    close D;
    @list = reverse(sort(@list));
    ( my $first_kernel = shift(@list) ) =~ s|win4lin-||;
    system("perl /usr/share/win4lin-loader/make-initrd $first_kernel") unless $debug;
    return $first_kernel;
}

sub do_check {
    my ($label, $version, $remove) = @_;
    if ($remove && !$label){
	print("not removing entry, entry doen't exists\n");
	exit;
    }
    if ($label && !$remove) {
	print("not adding entry, $label already exists\n");
	exit;
    }
    unless (-f "/boot/win4lin-$version") {
	print("look like i cannot find the specified kernel /boot/win4lin-$version\n") ;
	exit;
    }
}

sub symlinkf { unlink $_[1]; symlink $_[0], $_[1] }
sub cat_ { local *F; open F, $_[0] or $_[1] ? die "cat of file $_[0] failed: $!\n" : return; my @l = <F>; wantarray ? @l : join '', @l }

sub getoptions {
    my $lilo = shift;
    my ($options, $foo);
    my $quiet = getquiet_from_themes();
    my %var = getVarsFromSh("$sysconfig");
    my $cmdline = grep(/NOCMDLINE/, $var{OPTIONS}) ? undef : cat_("/proc/cmdline");chomp $cmdline;
    foreach $foo (split(/\s+/, join(" ", $var{OPTIONS}, $cmdline)))  {
	next if $foo =~ /auto/ and $lilo;
	next if $foo =~ /root=/;
	next if $foo =~ /quiet/ and $quiet;
	next if $foo =~ /^r[ow]/;
	next if $foo =~ /BOOT_IMAGE=/;
	next if $foo =~ /NOCMDLINE/;
	next if (grep /$foo/, $options);
	$options .= "$foo ";
    }
    return $options;
}


#Getroot(): First try to get root= from sysconfig files, if we don't
#	found it try to discover in the /proc/cmdline, if there is not
#	(like when booting from lilo that put the root in hexa
#	numbers) try to detect from fstab and if we didn't find it (no
#	s how) get it from rdev.

sub getroot {
    my $root;
    $root = getroot_from_sysconfig();
    $root = (grep /root=\/\S+/, split(" ", cat_('/proc/cmdline')))[0] unless $root;
    $root = getroot_from_fstab() unless $root;
    ($root = `rdev`) =~ s|(\S+)\s+.*$|root=$1| unless $root;chomp $root;
    warn("There is no root= defined you are going in trouble\n") unless $root;
    return $root;
}

sub getroot_from_sysconfig {
    my %var = getVarsFromSh("$sysconfig");
    return (grep /root=\/\S+/, split(" ", $var{OPTIONS}))[0];
}

sub gettheme_from_sysconfig {
    my %var = getVarsFromSh($sysconfig_theme);
    return ($var{THEME} ? $var{THEME} : "Mandrake");
}

sub getquiet_from_themes {
    return unless -f $sysconfig_theme;
    my $theme_config = "/usr/share/bootsplash/themes/" . gettheme_from_sysconfig() . "/global.config";
    my %var = getVarsFromSh($theme_config);
    return 1 if $var{QUIET} =~ /^yes$/i;
    return;
}

sub getroot_from_fstab {
    my $part;
    local *F;
    open F, '/etc/fstab';
    while (<F>) {
	my @s = split ' ';
	$part = $s[0] if $s[1] =~ m|/$| and not $part;
    }
    close F;
    return "root=" . $part;
}

sub getVarsFromSh(%) {
    my %l;
    local *F; open F, $_[0] or return;
    local $_;
    while (<F>) {
	s/#.*//; # remove comments
	my ($v, $val, $val2) =
	  /^\s*			# leading space
	   (\w+) =		# variable
	   (
	       "([^"]*)"	# double-quoted text
	     | '([^']*)'	# single-quoted text
	     | [^'"\s]+		# normal text
	   )
	   \s*$			# end of line
	  /x or next;
	$l{$v} = defined $val2 ? $val2 : $val;
    }
    %l;
}

sub get_kernel_type {
    $_ = shift;
    return $1 if (/.*($mdksub)$/);
    return undef;
}

my $pre = "pre|rc";

# sanitize_ver: long function when it could be shorter but we are sure
#		to catch everything and can be readable if we want to
#		add new scheme name.
sub sanitize_ver {
    my $string=shift;
    my $return;

    (my $kversion, my $kpatch, my $sublevel, my $extraversion, my $rest);


    $string =~ m|(\d+)\.(\d+)\.(\d+)(-((${pre})\d+))?(.*)|; #
    $kversion=$1; $kpatch=$2; $sublevel=$3;
    $extraversion=$5; $rest=$7;
    # $4 and $6 are garbage

    if ($rest =~ m|-(.+)-(.+)-(.+)-(.+)|) {
      my ($mdkver, $cpu, $nproc, $mem);
      $mdkver=$1;
      $cpu=$2;
      $nproc=$3;
      $mem=$4;
      $rest="$cpu$nproc$mem-$mdkver";
    }

    $return="$kversion$kpatch$sublevel$extraversion$rest" . "w4l";

    $return =~ s|\.||g;
    $return =~ s|mdk||;
    $return =~ s|64GB|64|;
    $return =~ s|4GB|4|;
    $return =~ s|secure|sec|;
    $return =~ s|enterprise|ent|;
    $return =~ s|smp|s|;
    $return =~ s|up|u|;
    $return =~ s|i686|686|;

    return $return;
}

1;#wonderful perl
